program ATEUPD,1.1(107)	 ! ASHW32 Updater
!---------------------------------------------------------------------------
!{Edit History}
!
! 23-06-2008 --- 1.1.108 --- Jack McGregor
! Support /V:51 switch to force update to 5.1, add edtFTPnet.dll, 
! workaround bug in 5.0 returning error 13 for launch
!
! 23-06-2008 --- 1.1.107 --- Jack McGregor
! Fix 5.1 problem - was only looking at 5.0 releases; also correct display
! of current version (was saying "ATE 50.") instead of "ATE 05.00");
!
! 01-02-2008 --- 1.1.106 --- Jack McGregor
! Support ATE 5.1.  (Set min/max versions for 5.0 to 5.0, for 5.1 to 5.1)
!
! 19-09-2007 --- 1.1.105 --- Jack McGregor
! Adjust to new UpdateCenter version format (uses dots rather than commas)
!
! 26-04-2007 --- 104 --- Jack McGregor
! Minor corrections/improvements in messagebox messages; and automatically copy UpdateCenter.exe from PC to server if not on server
!
! 24-04-2007 --- 103 --- Jack McGregor
! If we fail to connect to update center, report useful message.
!
! 22-04-2007 --- 102 --- Jack McGregor
! Replace PROGR~1 with Program Files and MI9373~1 with MicroSabio in messages (in case client is pre 987).
!
! 21-04-2007 --- 101 --- Jack McGregor
! Minor improvements, make work with ATSD
!
!
! 20-Apr-2007 --- 100 --- Jack McGregor
! Created based on ASHUPD
!---------------------------------------------------------------------------
!NOTES
! Decides if an ATE update is desirable and available, and if so,
! launches the Update Center to do it.   Based on new version
! of Update Center and ASHUPD.  Main differences with ASHUPD is that
! we are running from the server side, and spend a lot more effort
! on a framework for deciding whether an update is desirable in advance
! of actually launching the Update Center.
!
!SERVER SIDE SETUP:
!   You must pre-install into %MIAME%/atesetup (case sensitive names!):
!       UpdateCenter.exe
!       UpdateCenter.ini
!       ICSharpCode.SharpZipLib.dll
!	edtFTPnet.dll			! [108]
!---------------------------------------------------------------------------

++include ashinc:ashell.bsi
++include ashinc:ashell.def
++include sosfunc:fnverate.bsi		! [106] get ATE version

! Location of release notes html doc
define REL_NOTES50$ = "http://www.microsabio.net/dist/50rel/ate/atedevnotes.txt"
define PKG_NAME50$ = "ate"

![106]
define REL_NOTES51$ = "http://www.microsabio.net/dist/51rel/ate/atedevnotes.txt"
define PKG_NAME51$ = "ate-51"

define TRUE = -1

map1 PARAMS
    map2 GUIFLAGS,B,4
    map2 UPD$,S,1
    map2 STATUS,F
    map2 MIAME$,S,200
    map2 ATEDIR$,S,200
    map2 ELAPSE,F
    map2 RC,B,1
    map2 VERSTR$,S,28
    map2 X,F
    map2 Y,F
    map2 CRLF$,S,2,chr(13)+chr(10)
    map2 UPD'STATUS,I,1
    map2 MIN'VER$,S,14      ! min required version (MM.mm.eeee.pp)
    map2 MAX'VER$,S,14      ! max required version (MM.mm.eeee.pp)
    map2 ATE'VER$,S,14      ! actual ate version (MM.mm.eeee.pp)
    map2 AVL'VER$,S,14      ! available version (MM.mm.eeee.pp)
    map2 PROCEED,I,1
    map2 MBTNHELP,B,2
    map2 MHELP$,S,80
    map2 DIRSEP$,S,1
    map2 MSGAVL$,S,80
    map2 PKGNAME$,S,20	    ! [106]
    map2 RELNOTES$,S,200    ! [106]
    map2 SW$,S,20           ! [108]
    map2 SW1$,S,10          ! [108]
    map2 UPDVER$,S,4        ! [108]

!    xcall GTLANG,LANG'STATUS,GTLANG'MAP ! GTLANG'MAP in ashell.bsi

    xcall GETVER,VERSTR$                ! [108]
    ? tab(-1,0);"ATE Update ";VERSTR$   ! [108]
    xcall NOECHO

    ! First see if we are running ATE
    xcall AUI,AUI_ENVIRONMENT,MXOP_GET,GUIFLAGS
    if ((GUIFLAGS and AGF_ATE) = 0) then
        ? "Sorry, this only makes sense from an ATE connection."
        ? "(GUI flags: "+str(GUIFLAGS);")"
        End
    endif

    ! get MIAME env var into variable (SHELLEX will convert it for the
    ! executable name but not for the argument list)
    xcall MIAMEX,MX_GETENV,"MIAME",MIAME$
    if MIAME$ = "" then
        ? "Unable to proceed: MIAME environment variable not defined"
        End
    endif

    ! [108] check for command line switches (/V:51)
    X = instr(1,CMDLIN$,"/")
    do while X > 0
        ! pick up next /switch
        SW$ = CMDLIN$[X+1,-1]
        ! terminate it at first space, tab, ; or /
        Y = instr(1,SW$," ")
        if Y > 0 then SW$ = SW$[1,Y-1]
        Y = instr(1,SW$,";")
        if Y > 0 then SW$ = SW$[1,Y-1]
        Y = instr(1,SW$,chr(9))
        if Y > 0 then SW$ = SW$[1,Y-1]
        Y = instr(1,SW$,"/")
        if Y > 0 then SW$ = SW$[1,Y-1]
        ! compare it against the valid ones
        !trace.print SW$
        SW1$ = SW$                                  
        Y = instr(1,SW$,":")                        
        if Y > 0 then SW1$ = SW$[1,Y-1]             

        switch SW1$
            case "V"                                
                UPDVER$ = SW$[3,-1]
                exit
            case "VER"                              
                UPDVER$ = SW$[5,-1]
                exit

            default
                ? "Unsupported switch: ";SW$
                goto HELP
        endswitch
        ! check for next switch
        X = instr(X+1,CMDLIN$,"/")
    loop


    ! Determine if we want to, need to, and/or are able to update
    ! First get our min and max desired ATE versions...
    call Get'Min'Max'ATE'Requirements(MIN'VER$, MAX'VER$, UPDVER$)  ![108]
    ! Now compare that to our current and available versions
    ? "Checking version status, update availability..."
    UPD'STATUS = FN'Get'ATE'Update'Status(MIN'VER$, MAX'VER$, &
			  ATE'VER$, AVL'VER$, UPDVER$)     ![108]

    ? "Current ATE Version:            ";ATE'VER$
    ? "Version Available for Download: ";AVL'VER$
    ? "Minimum desired version:        ";MIN'VER$
    ? "Maximum desired version:        ";MAX'VER$

    ![106] set package name
    if (ATE'VER$[1,5]="05.01" and UPDVER$="") or UPDVER$="51" then ! [107][108]
	PKGNAME$ = PKG_NAME51$
	RELNOTES$ = REL_NOTES51$
    else
	PKGNAME$ = PKG_NAME50$
	RELNOTES$ = REL_NOTES50$
    endif

    switch UPD'STATUS
        case 0
            ![106] ? "ATE already fully updated"
 	    ? ucs(PKGNAME$);" already fully updated"	! [106]
            PROCEED = 0
            exit
        case 1
            if AVL'VER$[1,1]="?" then       ! [103] unknown update availability
                ![106] MSGAVL$ = "An ATE update MAY be available."                  ! [103]
                MSGAVL$ = "An ATE "+ATE'VER$[1,5]+" update MAY be available."  ! [103][106][107]
            else
                ![106] MSGAVL$ = "An ATE update is both desired and available."    ! [103]
                MSGAVL$ = "An ATE "+ATE'VER$[1,5]+" update is both desired and available."    ! [103][106][107]
            endif
            ? MSGAVL$;".."
            PROCEED = 1
            exit
        case 2
            MSGAVL$ = "ATE "+ATE'VER$[1,5]+" update required, proceeding."    ! [103][106][107]

            ? MSGAVL$;"..";
            PROCEED = 1
            exit
        case 3
            ? "ATE ";ATE'VER$[1,5];" version already higher than desirable" ![106][107]
            PROCEED = -2
            exit
        case 9
            !? "ATE update desired but not available"
            ? "ATE ";ATE'VER$[1,5];" update allowed, but current version matches latest available" ![107]
            PROCEED = -1
            exit
        case 10
            ? "ATE ";ATE'VER$[1,5];" update required but not available" ![107]
            PROCEED = -1
            exit
        case 17
            ? "ATE ";ATE'VER$[1,5];" update desired but Update Center not accessible" ![107]
            PROCEED = -1
            exit
        case 18
            ? "ATE ";ATE'VER$[1,5];" update required but Update Center not accessible" ![107]
            PROCEED = -1
            exit
        default
            ? "Unexpected ATE Update status: ";UPD'STATUS
            PROCEED = -3
            exit
     endswitch

    if PROCEED <= 0 goto ENDIT      ! no point in trying to update these

    ! Else proceed to try to update

    ! We need to translate %ATE% because MX_SHELLEX won't do it for the parms arg
    ! (But use MIAME instead, since there is no ambiguity that we are talking about
    ! the PC environment, and ATE was a recent addition)
    ? tab(-10,AG_GETENV);"MIAME";chr(127);
    input "",ATEDIR$


    ! Make sure the local ATE machine has the necessary modules, downloading
    ! them if necessary.
    ! get directory separator character
    xcall MIAMEX, MX_DIRSEP, DIRSEP$

    if FN'UpdateLocalCopy(MIAME$+DIRSEP$+"atesetup","UpdateCenter.exe",ATEDIR$+"\bin") goto NEEDUPC
    if FN'UpdateLocalCopy(MIAME$+DIRSEP$+"atesetup","UpdateCenter.ini",ATEDIR$+"\bin") goto NEEDUPC
    if FN'UpdateLocalCopy(MIAME$+DIRSEP$+"atesetup","ICSharpCode.SharpZipLib.dll",ATEDIR$+"\bin") goto NEEDUPC
    if FN'UpdateLocalCopy(MIAME$+DIRSEP$+"atesetup","edtFTPnet.dll",ATEDIR$+"\bin") goto NEEDUPC ![108]

    ! Now we have the UpdateCenter files on the PC, warn user of what is about to happen...

    do
        ! In order to avoid bug in pre-987 ATE in which MBTN_HELP would cause sign extension
        ! (and not work), we only specify it if we have a later version.
        if ATE'VER$[7,10] >= "0987" then
            MBTNHELP = MBTN_HELP
            MHELP$ = CRLF$ + CRLF$ +"Click HELP to display release notes."
        else
            MBTNHELP = 0
            MHELP$ = ""
        endif


       xcall MSGBOX,MSGAVL$ + CRLF$ + CRLF$ + "In order to proceed, the terminal " &
            + "session will be terminated during the update and will be restarted " &
            + "when the process is complete, which should be no more than a minute or two. " &
            + CRLF$ + CRLF$ &
            + "(You'll have to manually close any other instances of ATE before proceeding.)" &
            + MHELP$, &
            "ATE Update Center", MBTN_OK_CANCEL+MBTNHELP, 0, 0, RC

        if RC = MBRC_HELP then
            xcall MIAMEX, MX_SHELLEX, STATUS, RELNOTES$		![106]
        endif
    loop until RC # MBRC_HELP

    if RC = MBRC_OK then

        xcall MIAMEX,MX_SHELLEX, STATUS, ATEDIR$+"\bin\UpdateCenter.exe", &
    		"",PKGNAME$ + " -D"""+ATEDIR$ +""""	![106]
        if STATUS = 0 or STATUS = 13 then	! [108] (workaround old 5.0 bug)
!            xcall MIAMEX,MX_EXIT               ! close this instance of A-Shell
             xcall SLEEP,1
             ? tab(-10,AG_ATEDISCONNECT);chr(127);
        else
            ? "Unable to launch Update Center - Error ";STATUS
        endif
    else
        ? "Update aborted"
    endif

ENDIT:
    End

HELP:           ! [108] display switches
    ? "ATEUPD Switches:"
    ? 
    ? "    /V:51 - Force update to version 5.1 (from 5.0)"
    ?
    end

NEEDUPC:        ! if we failed to transfer the UpdateCenter files, give the user
                ! a chance to do it manually

    ! [102] try to at least partially clean up stupid directory name
    ! [102] (Note: this shouldn't be a problem after 5.0.987)
    X = instr(1,ucs(ATEDIR$),"PROGRA~1")
    if X > 1 then
        ATEDIR$ = ATEDIR$[1,X-1] + "Program Files" + ATEDIR$[X+8,-1]
    endif
    X = instr(1,ucs(ATEDIR$),"MI9373~1")
    if X > 1 then
        ATEDIR$ = ATEDIR$[1,X-1] + "MicroSabio" + ATEDIR$[X+8,-1]
    endif


    xcall MSGBOX,"""We were unable to automatically install the Update Center modules needed on your PC " &
        + "in order to perform the update.  If you would like to download and install those modules " &
        + "yourself, click YES below.  This should bring up a browser window giving you the option to " &
        + "save or open UpdateCenter.zip (or perhaps will just save it automatically, depending on your " &
        + "browser configuration).  The goal is to OPEN the zip and extract the three files into the " &
        + "following directory: " + CRLF$ + CRLF$ + ATEDIR$ + "\BIN" + CRLF$ + CRLF$ &
        + "Once you have done that, re-run ATEUPD to perform the update. " &
        + "(The Update Ceneter modules will be copied to the server for distribution to other ATE clients so " &
        + "they don't all have to go through the manual procedure.)"+ CRLF$ + CRLF$ &
        + "Would you like to perform the download now? """, &
        "ATEUPD Update Problem Work-around", &
        MBTN_YES_NO, MBICON_QUESTION, MBMISC_TASKMODAL, RC    ! [102]
    if RC = MBRC_YES then
        xcall MIAMEX, MX_SHELLEX, STATUS, "http://updatecenter.microsabio.net/UpdateCenter.zip"
        if STATUS # 0 then
            xcall MSGBOX,"Unable to launch browser to perform download. " &
                + "Try manually browsing to http://updatecenter.microsabio.net/UpdateCenter.zip", &
                "ATEUPD Update Problem Work-around", MBTN_OK, MBICON_ICON, MBMISC_TASKMODAL, RC
        endif
    endif
    End

!----------------------------------------------------------------------------------------------
! Check if a specified file is on the remote machine and matches the one on the local
! machine, and if not, download it (if we have it).
! Returns 0=ok, else error
!NOTES:
!   Detects Windows and uses the alternate file transfer method if so
!----------------------------------------------------------------------------------------------

Function FN'UpdateLocalCopy(ldir$ as S200, file$ as s60, rdir$ as s200) as i1

    map1 hostbytes,f
    map1 pcbytes,f
    map1 a,f
    map1 rc,b,1
    Static map1 dirsep$,s,1,""

    ! get directory separator character
    if dirsep$ = "" then
        xcall MIAMEX, MX_DIRSEP, dirsep$
    endif

    ! See if we have the local copy...
    xcall MIAMEX,MX_FILESTATS,"L",ldir$+dirsep$+file$,hostbytes

    ! See if the file is present in the %ate%\bin directory
    xcall MIAMEX,MX_FILESTATS,"R",rdir$+"\"+file$,pcbytes

    ! If they differ and we have a local copy, download it
    if (pcbytes=-1 and hostbytes>0) or (hostbytes>0 and hostbytes#pcbytes) then

        if dirsep$ = "/" then       ! if UNIX, assume FTP avail
            ? tab(-10,AG_FTP);"0";ldir$;dirsep$;file$;chr(126);rdir$;"\";file$;chr(127);
            xcall NOECHO
            xcall CCOFF
            xcall ACCEPN,a

        else                        ! if Windows, use aux port method

            ! Send file via expanded aux port protocol
            ! First send the header info
            ? tab(-10,AG_SPOOLCFG);
            ? ".\";                     ! force it to be native
            ? file$;                    ! file.ext
            ? ",DISK:";rdir$;           ! use pseudo printer DISK:<path>
            ? ",";str(hostbytes);       ! binary flag (size)
            ? chr(127);

            ? tab(-1,82);               ! open aux port

            ! use B64ENC to output file
            xcall B64ENC,ldir$+dirsep$+file$,0,a

            ! adjust status to match ftp status
            if a < 0 then
                a = 3       ! error
            else
                a = 13      ! ok
            endif
            ? tab(-1,83);               ! close aux port

        endif

        if a = 13 then
            ! double check to see if it worked
            xcall MIAMEX,MX_FILESTATS,"R",rdir$+"\"+file$,pcbytes
            a = 13
            if pcbytes # hostbytes then
                a = 3
            endif
        endif

        if a # 13 then
            ! note double-quoting of message in case there are tildes in the directories
            xcall MSGBOX,"""Unable to transfer "+ldir$+dirsep$+file$+" to "+rdir$+"""", &
                "ATE Update Error", MBTN_OK, MBICON_STOP, 0, rc
            rc = 1  ! file transfer failure
        endif
    else
        if (pcbytes <= 0) then
            xcall MSGBOX,"""Cannot proceed; file  "+rdir$+"\"+file$+" not present on PC """ &
                + "or on host (in "+ldir$+") in order to be transferred.", &
                "ATE Update Error", MBTN_OK, MBICON_STOP, 0, rc
            rc = 2  ! file not present

        else
            ! [104] If we have the remote copy but not the local copy, upload it (if UNIX)
            if hostbytes=0 and dirsep$="/" then       ! if UNIX, assume FTP avail
                ? tab(-10,AG_FTP);"1";ldir$;dirsep$;file$;chr(126);rdir$;"\";file$;chr(127);
                xcall NOECHO
                xcall CCOFF
                xcall ACCEPN,a
            endif
        endif
    endif
    xcall CCON
    xcall ECHO
    FN'UpdateLocalCopy = rc
End Function

!----------------------------------------------------------------------------------------------
! Determine the minimum and maximum desired ATE versions.  This would be up to the
! application, dealer, or site, and probably set by a sysop and stored in some
! file.
! Parameters:
!   min'ver$    [string, out]   MM.mm.eeee.pp  (M=major, m=minor, e=edit, p=patch)
!   max'ver$    [string, out]   MM.mm.eeee.pp  (M=major, m=minor, e=edit, p=patch)
!   upd'to'ver$ [string, in]    "51" or "50" or "" (use to force version) [108]
!   
! Notes:
!   We now use the current ATE version (5.0 or 5.1) to determine the min
!   and max allowed versions.
!----------------------------------------------------------------------------------------------

Procedure Get'Min'Max'ATE'Requirements(min'ver$ as s14, &
					max'ver$ as s14, &
					upd'to'ver$ as s4) ![108]

    map1 locals			! [106]
        map2 ate'ver$,s,14	! [106]

    ate'ver$ = Fn'VerATE$()	! [106]

    ! TO DO: (customize)

    ! For now, just set the min and max to defaults
    ! [106] but within the current vmajor.vminor
    if ((ate'ver$[1,4]="5.1." and upd'to'ver$="") or (upd'to'ver$="51"))  then       ! [106][108]
        min'ver$ = "05.01.1100.00"         ! [106]
        max'ver$ = "05.01.9999.99z"        ! [106][107]
    else
        min'ver$ = "05.00.0987.00"
        max'ver$ = "05.00.9999.99z"
    endif
    xputarg 1, min'ver$
    xputarg 2, max'ver$

End Procedure

!----------------------------------------------------------------------------------------------
! Compare the current, min, max, and available versions and return a status value
! summarizing what to do.
!   0 = no update needed or desired
!   1 = update desired
!   2 = update required
!   3 = already past max desired version
!  +8 = desired version not available on Update Center
! +16 = Update Center not accessible (no internet?)
!
! Parameters:
!   min'ver$    [string, in]   Min required version (MM.mm.eeee.pp)
!   max'ver$    [string, in]   Max required version (MM.mm.eeee.pp)
!   ate'ver$    [string, out]  Current ATE version (MM.mm.eeee.pp)
!   avl'ver$    [string, out]  Version available for download (MM.mm.eeee.pp)
!   upd'to'ver$ [string, in]  "51" or "50" or "" [108] (use to force ver)
!
!-----------------------------------------------------------------------------------------------

Function FN'Get'ATE'Update'Status(min'ver$ as s14, max'ver$ as s14, &
                                  ate'ver$ as s14, avl'ver$ as s14, &
				  upd'to'ver$ as s4)

    map1 locals
        map2 verstr$,s,28
        map2 vmajor,b,1
        map2 vminor,b,1
        map2 vedit,b,2
        map2 vpatch,b,1
        map2 ate'vmajor,b,1
        map2 ate'vminor,b,1
        map2 ate'vedit,b,2
        map2 ate'vpatch,b,1
        map2 status,i,1

    xcall MIAMEX, MX_GETVER, verstr$, vmajor, vminor, vedit, vpatch, &
                ate'vmajor, ate'vminor, ate'vedit, ate'vpatch

    ate'ver$ = (ate'vmajor using "#Z") + "." + (ate'vminor using "#Z") &
        + "." + (ate'vedit using "#ZZZ") + "." + (ate'vpatch using "#Z")

    xputarg 3,ate'ver$

    avl'ver$ = FN'Get'Avail'ATE'Version$("updatecenter.microsabio.net", &
			ate'ver$, upd'to'ver$) ![107][108]

    xputarg 4,avl'ver$

    Debug.print "ate'ver$ = " + ate'ver$
    Debug.print "min'ver$ = " + min'ver$
    Debug.print "max'ver$ = " + max'ver$
    Debug.print "avl'ver$ = " + avl'ver$

    ! Now synthesize the situation into one status value
    ! First, based on comparison of actual and min/max values
    if ate'ver$ = max'ver$ then status = 0      &
    else if ate'ver$ < min'ver$ then status = 2 &
    else if ate'ver$ < max'ver$ then status = 1 &
    else status = 3

    ! Now factor in the availability
    switch status
        case 0          ! available doesn't matter; we have the ideal version
        case 3          ! we have too high a version already
            exit

        case 1          ! we desire an update
        case 2          ! we require an update
            if avl'ver$ = "" then        &
                status = status + 16     &
            else if avl'ver$ <= ate'ver$ &
               status = status +8
            exit
    endswitch

    FN'Get'ATE'Update'Status = status

End Function


!----------------------------------------------------------------------------------------------
! Connect to Update Center to determine what version is available
! Params
!   hostname$ [in] - base URL where updatecenter repository is
!   cur'ate'ver$ [in] (MM.mm.eeee.pp) current version (to get pkg from)
!   upd'to'ver$  [in] "50" or "51" or "" (to override version)
! Returns:
!   avl'ver$ (MM.mm.eeee.pp) or "" if we can't connect
!       special case: if we connect but can't parse the version info,
!       we return "?" which which be above all version numbers and
!       thus cause caller to probably try to do update anyway
! Parameters:
!   hostname$ [in,string]
!----------------------------------------------------------------------------------------------


Function FN'Get'Avail'ATE'Version$(hostname$ as s80, &
				   cur'ate'ver$ as s14, &
				   upd'to'ver$ as s4) as s14

    map1 tcp'params
        map2 tcp'timer,f,6,5000
        map2 tcp'sts,f,6
        map2 tcp'sockport,f
        map2 tcp'buffer$,s,512
        map2 x,f
        map2 crlf$,s,2,chr(13)+chr(10)
        map2 linebuf$,s,100
        map2 sepver$,s,20
        map2 vmajor$,s,4
        map2 vminor$,s,4
        map2 vedit$,s,8
        map2 vpatch$,s,4
        map2 fdelim$,s,2
        map2 rdelim$,s,2
        map2 op,b,1
        map2 fdate$,s,32
        map2 fsize$,s,14
        map2 fzipsize$,s,14
        map2 rc,b,1
	map2 pkgname$,s,20		! [107]

    ! Here we try to connect to the Update Center on the internet to
    ! determine what's available.

    Debug.print "Connection to "+hostname$
    tcp'sockport = 80
    xcall TCPX,TCPOP_CONNECT,tcp'sts,tcp'buffer$,tcp'sockport,TCPXFLG_BLOCK, &
            tcp'timer,hostname$
    Debug.print "status = " + tcp'sts + ", socket = " +tcp'sockport
    if tcp'sts < 0 or tcp'sockport < 0 then goto tcperror   ! [103]

    if upd'to'ver$ = "51" then                  ! [108]  
	pkgname$ = PKG_NAME51$
    endif
    if upd'to'ver$ = "50" then
	pkgname$ = PKG_NAME50$
    endif
    
    if pkgname$ = "" then                       ! [108] 
        if cur'ate'ver$ > "05.00.zzzz" then	! [107]
            pkgname$ = PKG_NAME51$
    	else
            pkgname$ = PKG_NAME50$
        endif
    endif

    tcp'buffer$ = "GET /"+pkgname$+"/filesv3.txt HTTP/1.1" + chr(13) + chr(10) &
        + "Host: updatecenter.microsabio.net:80" + chr(13) + chr(10) &
        + chr(13) + chr(10)  ![107]

    Debug.print tcp'buffer$[1,-3]
    xcall TCPX, TCPOP_WRITE, tcp'sts, tcp'buffer$, tcp'sockport,  &
        len(tcp'buffer$), tcp'timer
    Debug.print "status = " + tcp'sts
    if tcp'sts <= 0 goto tcperror

    ! now read it looking for ASHW32.EXE= line
    do
        Debug.print "read..."
        tcp'buffer$ = ""
        xcall TCPX, TCPOP_READ, tcp'sts, tcp'buffer$, tcp'sockport, 0, tcp'timer
        Debug.print "status = " + tcp'sts + " [" +tcp'buffer$ + "]"
        ! See if we have the Version=##.####.##.## line
!        x = instr(1,tcp'buffer$,"Version=")
!        if x > 0 then
!            FN'Get'Avail'ATE'Version$ = tcp'buffer$[x+8;13]
!            exit
!        endif

        ! since tcp'buffer$ may contain multiple logical lines, we have to
        ! unpack it
        do while tcp'buffer$ # ""

            x = instr(1,tcp'buffer$,chr(10))       ! find line terminator

            if x > 0 then                          ! ok we have complete line

                linebuf$ = linebuf$ + tcp'buffer$[1,x]
Debug.print "linebuf$ = "+linebuf$

                ! Example:
                ! ASHW32.exe=2007-04-20 16:45:58|1683032|868067|5, 0, 987, 0|bin|BIN

                if ucs(linebuf$[1,11]) = "ASHW32.EXE=" then

                    ! parse it into pieces
                    op = 0
                    fdelim$ = "|"
                    rdelim$ = ""
                    xcall STRTOK, op, linebuf$, fdelim$, fdate$, rdelim$, &
                            fsize$, fzipsize$, sepver$

DEBUG.PRINT "FN'Get'Avail'ATE'Version$: op=["+op+"] linebuf$=["+linebuf$+"] fdelim$=["+fdelim$+"] fdate$=["+fdate$+"] rdelim$=["+rdelim$+"] " &
    + " fsize$=["+fsize$+"] fzipsize$=["+fzipsize$+"] sepver$=["+sepver$+"] "

                    if sepver$ = "" then
                        xcall MSGBOX, """Unable to parse Update Center line format in filesv3.txt: " &
                           + crlf$ + crlf$ + tcp'buffer$[1,x] +"""", &
                            "ATEUPD Error", MBTN_OK, MBICON_EXCLAMATION, 0
                        FN'Get'Avail'ATE'Version$ = "?"

                    else
                        ! parse the version string
                        op = 0
                        fdelim$ = ",."      ! [106] add dot to field delimeters
                        rdelim$ = "|"
                        xcall STRTOK, op, sepver$, fdelim$, vmajor$, rdelim$, vminor$, vedit$, vpatch$
DEBUG.PRINT "FN'Get'Avail'ATE'Version$: op=["+op+"] sepver$=["+sepver$+"] fdelim$=["+fdelim$+"] vmajor$=["+vmajor$+"] rdelim$=["+rdelim$+"] vminor$=["+vminor$+"] vedit$=["+vedit$+"] vpatch$=["+vpatch$+"] "

                        ! now reformat it
                        FN'Get'Avail'ATE'Version$ = (val(vmajor$) using "#Z") + "." &
                            + (val(vminor$) using "#Z") + "." &
                            + (val(vedit$) using "#ZZZ") + "." &
                            + (val(vpatch$) using "#Z")

                    endif
                    linebuf$ = ""
                    exit        ! we can quit after reading the ASHW32.EXE line
                                ! (in the future, we might want to scan for other info)
                endif
                linebuf$ = ""
                tcp'buffer$ = tcp'buffer$[x+1,-1]


            else    ! incomplete line; save partial in linebuf

               linebuf$ = linebuf$ +tcp'buffer$
Debug.print "linebuf$ (partial) = "+linebuf$
               tcp'buffer$ = ""
            endif

        loop

        if FN'Get'Avail'ATE'Version$ # "" then exit     ! quit once we get what we want

    loop until tcp'sts=0 or tcp'sts<0

    Debug.print "close connection, ver = " + FN'Get'Avail'ATE'Version$
    xcall TCPX, TCPOP_CLOSE, tcp'sts, tcp'buffer$, tcp'sockport, 0

    Exitfunction

tcperror:
    ! [103] display a message if we fail to connect and give operator option to continue anyway
    xcall MSGBOX,"Unable to connect to http://updatecenter.microsabio.net" &
        + crlf$ + crlf$ + "This could be because the Update Center is down, or it may be that " &
        + "you don't have outbound Internet access from this server, in which case you may still " &
        + "be able to connect from your PC.  If you think the latter might be the case and want " &
        + "to try to update anyway, click YES.  Else click NO to terminate the procedure.", &
        "ATEUPD Update Problem Work-around", MBTN_YES_NO, MBICON_ICON, MBMISC_TASKMODAL, rc
    if rc = MBRC_YES then
        FN'Get'Avail'ATE'Version$ = "?"     ! this will appear as a higher version than we have now
    else
        FN'Get'Avail'ATE'Version$ = ""
    endif

End Function


